/**
 * PETALS: PETALS Services Platform Copyright (C) 2005 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: ComponentLifeCycle.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.container.lifecycle;

import java.io.File;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.jbi.JBIException;
import javax.jbi.component.Component;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.jndi.client.JNDIService;
import org.ow2.petals.container.thread.ComponentLifeCycleThread;
import org.ow2.petals.jbi.component.context.ComponentContextImpl;
import org.ow2.petals.jbi.component.context.ComponentInitialContext;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.jbi.management.task.installation.InstallationUtils;
import org.ow2.petals.jbi.messaging.endpoint.ConsumerEndpoint;
import org.ow2.petals.jbi.messaging.exchange.DeliveryChannelImpl;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * Implementation of the <code>ComponentLifeCycle</code> mbean
 * 
 * @author Adrien LOUIS - EBM WebSourcing
 * @author wjoseph - eBM-WebSourcing
 * @author nsalatge - eBM-WebSourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.container.lifecycle.ComponentLifeCycleMBean.class))
public class ComponentLifeCycle extends LifeCycleAbstract implements ComponentLifeCycleMBean {

    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * Platform component :: SystemState Service
     */
    @Requires(name = "systemstate", signature = org.ow2.petals.platform.systemstate.SystemStateService.class)
    private SystemStateService systemStateService;

    /**
     * Communication component :: JNDIService
     */
    @Requires(name = "jndi", signature = org.ow2.petals.communication.jndi.client.JNDIService.class)
    protected JNDIService jndiService;

    /**
     * JBI-Management Component :: Admin Service
     */
    @Requires(name = "admin", signature = org.ow2.petals.jbi.management.admin.AdminServiceMBean.class)
    protected AdminServiceMBean adminService;

    /**
     * The Configuration service fractal component
     */
    @Requires(name = "configuration", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    /**
     * The container configuration
     */
    private ContainerConfiguration containerConfiguration;

    /**
     * The component linked with the current component life cycle
     */
    protected Component component;

    /**
     * The component description
     */
    private org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription;

    /**
     * The component context
     */
    private ComponentContextImpl componentContext;

    /**
     * The component life cycle thread. Each life cycle have a thread to process
     * operation in the component ClassLoader
     */
    protected ComponentLifeCycleThread componentLifeCycleThread;

    protected Jbi descriptor;

    protected String installationRootPath;

    protected ConsumerEndpoint address;

    protected InitialContext initialContext;

    protected Map<String, Object> context;

    /**
     * Creates a new instance of {@link ComponentLifeCycle}
     * 
     */
    public ComponentLifeCycle() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.container.lifecycle.ComponentLifeCycleMBean#init(org.ow2
     * .petals.jbi.descriptor.original.generated.Component,
     * org.ow2.petals.jbi.component.context.ComponentContextImpl,
     * javax.jbi.component.Component, javax.management.ObjectName,
     * org.ow2.petals.container.thread.ComponentLifeCycleThread,
     * org.ow2.petals.util.LoggingUtil, java.util.Map)
     */
    public void init(
            org.ow2.petals.jbi.descriptor.original.generated.Component componentDescription,
            ComponentContextImpl componentContext, Component component, ObjectName mbeanName,
            ComponentLifeCycleThread componentLifeCycleThread, LoggingUtil log,
            Map<String, Object> context) throws PetalsException {

        log.start();

        super.init(mbeanName, log);

        this.componentDescription = componentDescription;

        this.componentContext = componentContext;

        this.component = component;

        this.context = context;

        this.descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);

        URI installRoot = (URI) context.get(TaskContextConstants.INSTALL_ROOT);

        this.name = InstallationUtils.getComponentNameFromDescriptor(this.descriptor);

        this.installationRootPath = new File(installRoot).getAbsolutePath();

        this.address = new ConsumerEndpoint(this.name, this.containerConfiguration.getName(),
                this.containerConfiguration.getSubdomainName());

        this.initialContext = this.createComponentInitialContext();

        // life cycle thread
        this.componentLifeCycleThread = componentLifeCycleThread;

        log.end();
    }

    @Override
    public void doInit() throws JBIException {
        this.log.start();
        this.componentLifeCycleThread.doInit(this.componentContext);
        this.log.end();
    }

    @Override
    public void doShutdown() throws JBIException {
        this.log.start();
     // if a service unit (on any state) need this component, you can't shutdown
        // the component
        if (!getServiceUnitLifeCycles().isEmpty()) {
            throw new UnauthorizedJBIException(
                    "While a service unit (on any state) need this component, you can't stop the component. Please undeploy (if needed by stopping and then shutdowning) all the services assemblies linked to this component before to stop it.");
        }
        this.componentLifeCycleThread.doShutdown();
        // close the Channel if required
        DeliveryChannelImpl channel = this.componentContext.getDeliveryChannelImpl();
        if (channel != null && channel.isOpened())
            channel.close();
        this.log.end();
    }

    @Override
    public void doStart() throws JBIException {
        this.log.start();
        this.componentLifeCycleThread.doStart();
        boolean isFound = false;
        // in the case of a restart, we restart all the shutdown SU LifeCycle
        List<ServiceUnitLifeCycle> suLifeCycles = getServiceUnitLifeCycles();
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = adminService
                .getServiceAssemblyLifeCycles();
        for (ServiceUnitLifeCycle suLifeCycle : suLifeCycles) {
            if (SHUTDOWN.equals(suLifeCycle.getCurrentState())) {
                for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
                    List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnit();
                    for (ServiceUnit unit : sus) {
                        if (suLifeCycle.getSuName().equals(unit.getIdentification().getName())) {
                            isFound = true;
                            if (STARTED.equals(salc.getCurrentState())) {
                                suLifeCycle.init();
                                suLifeCycle.start();
                            } else if (STOPPED.equals(salc.getCurrentState())) {
                                suLifeCycle.init();
                            }
                            break;
                        }
                    }
                    if (isFound) {
                        isFound = false;
                        break;
                    }
                }

            }
        }
        this.log.end();
    }

    @Override
    public void doStop() throws JBIException {
        this.log.start();
        // if a service unit (on any state) need this component, you can't stop
        // the component
        if (!getServiceUnitLifeCycles().isEmpty()) {
            throw new UnauthorizedJBIException(
                    "While a service unit (on any state) need this component, you can't stop the component. Please undeploy (if needed by stopping and then shutdowning) all the services assemblies linked to this component before to stop it.");
        }
        this.componentLifeCycleThread.doStop();
        this.log.end();
    }

    /**
     * @see javax.jbi.management.ComponentLifeCycleMBean#getExtensionMBeanName()
     */
    public ObjectName getExtensionMBeanName() throws JBIException {
        return this.componentLifeCycleThread.getExtensionMBeanName();
    }

    /**
     * 
     * @return
     */
    public String getName() {
        return this.name;
    }

    @Override
    public void setState(String state) throws JBIException {
        super.setState(state);
        try {
            this.systemStateService.updateComponentLifeCycleState(this.name, state);
        } catch (Exception e) {
            String msg = "Component state can't be persisted";
            this.log.error(msg, e);
            throw new JBIException(msg, e);
        }
    }

    /* getters */

    public Component getComponent() {
        return this.component;
    }

    public org.ow2.petals.jbi.descriptor.original.generated.Component getComponentDescription() {
        return this.componentDescription;
    }

    /**
     * Get the logger
     * 
     * @return the logger
     */
    public Logger getLogger() {
        return this.logger;
    }

    /**
     * Initialization of the petals component
     * 
     * @throws PetalsException
     * @throws JBIException
     * 
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void startComponentLifeCycle() {
        this.log = new LoggingUtil(this.logger);
        this.log.call();
        this.containerConfiguration = this.configurationService.getContainerConfiguration();
    }

    /**
     * Stop of the petals component
     * 
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stopComponentLifeCycle() {
        this.log.start();

        // there is ambiguity between the STOP of the life-cycle and the STOP of
        // the fractal component, so stop the component only if the life-cycle
        // state is STARTED
        if (STARTED.equals(this.getCurrentState())) {
            try {
                this.doStop();
            } catch (JBIException e) {
                this.log.error("Failed to stop the component '" + this.name + "'", e);
            }
        }

        try {
            this.componentLifeCycleThread.shutdownThread();
        } catch (JBIException e) {
            this.log.error("Failed to stop the life-cycle dedicated thread for the component '"
                    + this.name + "'", e);
        }

        this.log.end();
    }

    /**
     * Get the list of the deployed SU on the component.
     * 
     * @return a list of {@code ServiceUnitLifeCycle}
     */
    public List<ServiceUnitLifeCycle> getServiceUnitLifeCycles() {
        List<ServiceUnitLifeCycle> suLifeCycles = new ArrayList<ServiceUnitLifeCycle>();
        for (ServiceAssemblyLifeCycle saLifeCycle : this.adminService
                .getServiceAssemblyLifeCycles().values()) {
            for (ServiceUnitLifeCycle suLifeCycle : saLifeCycle.getServiceUnitsLifeCycles()
                    .values()) {
                if (suLifeCycle.getTargetComponentName().equals(this.name)) {
                    suLifeCycles.add(suLifeCycle);
                }
            }
        }
        return suLifeCycles;
    }

    @Override
    protected String getJBIArtifactType() {
        return "Component";
    }

    /**
     * The initial JNDI context for all component. If this initial context is
     * null, it will be created
     * 
     * @return the initial JNDI context for all component. If this initial
     *         context is null, it will be created
     * @throws PetalsException
     *             if {@link InitialContext} can't be created
     */
    private InitialContext createComponentInitialContext() throws PetalsException {
        ComponentInitialContext componentInitialContext = null;
        try {
            componentInitialContext = new ComponentInitialContext(this.jndiService
                    .getUsersContext());
        } catch (NamingException e) {
            throw new PetalsException("Error creating InitialContext for components", e);
        }
        return componentInitialContext;
    }
}
