/**
 * PETALS: PETALS Services Platform Copyright (C) 2005 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: AutoLoaderServiceImpl.java,v 1.2 2006/04/07 10:24:27 alouis Exp $
 * --------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.autoload;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URI;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.Timer;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.communication.jmx.JMXService;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit;
import org.ow2.petals.jbi.descriptor.original.generated.Component.SharedLibrary;
import org.ow2.petals.jbi.management.deployment.DeploymentServiceMBean;
import org.ow2.petals.jbi.management.installation.InstallationServiceMBean;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.jbi.management.util.XMLResult;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.platform.repository.RepositoryServiceImpl.EntityType;
import org.ow2.petals.platform.systemstate.ComponentState;
import org.ow2.petals.platform.systemstate.ServiceAssemblyState;
import org.ow2.petals.platform.systemstate.SharedLibraryState;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

import static org.ow2.petals.kernel.api.server.PetalsServer.INSTALLED_DIRECTORY;
import static org.ow2.petals.kernel.api.server.PetalsServer.INSTALL_DIRECTORY;
import static org.ow2.petals.kernel.api.server.PetalsServer.WORK_DIRECTORY;

/**
 * This class is used to load automatically components which are placed in the
 * "autoLoad" directory. It scans the directory periodically and check if new
 * components are presents. The @ link InstallationServiceMBean} is called for
 * each of those components. The whole process of installation is done (install
 * + start) After being threat, the archives of the components are removed. A
 * file "fileName.success" will be created if success A file "fileName.failed"
 * will be created otherwise, containing the related exception
 * 
 * @author alouis
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.ow2.petals.jbi.management.autoload.AutoLoaderService.class))
public class AutoLoaderServiceImpl implements AutoLoaderService {

    private static final long scanPeriod = 4000L;

    private static File installDirectory;

    private static File installedDirectory;

    private static File workDirectory;

    protected Timer installDirectoryScanTimer;

    protected Timer installedDirectoryScanTimer;

    /**
     * The local container information.
     */
    private ContainerConfiguration containerConfiguration;

    @Monolog(name = "logger")
    protected Logger logger;

    protected LoggingUtil log;

    /**
     * Package Handler
     */
    protected PackageHelper packageHelper;

    @Requires(name = "configuration", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class)
    private ConfigurationService configurationService;

    @Requires(name = "jmx", signature = org.ow2.petals.communication.jmx.JMXService.class)
    protected JMXService jmxService;

    @Requires(name = "systemstate", signature = org.ow2.petals.platform.systemstate.SystemStateService.class)
    protected SystemStateService systemStateService;

    @Requires(name = "repository", signature = org.ow2.petals.platform.repository.RepositoryService.class)
    protected RepositoryService repositorySrv;

    @Requires(name = "deployment", signature = org.ow2.petals.jbi.management.deployment.DeploymentServiceMBean.class)
    protected DeploymentServiceMBean deploymentService;

    @Requires(name = "installation", signature = org.ow2.petals.jbi.management.installation.InstallationServiceMBean.class)
    protected InstallationServiceMBean installationService;

    /**
     * Initialize the autoloader.
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IOException {
        log = new LoggingUtil(logger);
        log.call();

        this.containerConfiguration = this.configurationService.getContainerConfiguration();
        installDirectory = new File(this.containerConfiguration.getInstallDirectoryPath()
                + File.separator + INSTALL_DIRECTORY);
        installedDirectory = new File(this.containerConfiguration.getInstallDirectoryPath()
                + File.separator + INSTALLED_DIRECTORY);
        workDirectory = new File(this.containerConfiguration.getInstallDirectoryPath()
                + File.separator + WORK_DIRECTORY);

        // old start method
        installDirectoryScanTimer = new Timer();
        installedDirectoryScanTimer = new Timer();
        InstallDirectoryScanner installScan = new InstallDirectoryScanner(this, installDirectory);
        InstalledDirectoryScanner installedScan = new InstalledDirectoryScanner(this,
                installedDirectory);
        /*
         * Initialize package handler
         */
        packageHelper = new PackageHelper(logger, repositorySrv);

        installDirectoryScanTimer.schedule(installScan, scanPeriod, scanPeriod);
        installedDirectoryScanTimer.schedule(installedScan, scanPeriod, scanPeriod);
    }

    /**
     * Stop the autoload of the components. TODO : synchronization, does the
     * autoload must wait before the end of an installation before closing?
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() {
        log.call();

        if (installDirectoryScanTimer != null) {
            installDirectoryScanTimer.cancel();
        }
        if (installedDirectoryScanTimer != null) {
            installedDirectoryScanTimer.cancel();
        }
    }

    /**
     * @see org.ow2.petals.jbi.management.autoload.AutoLoaderService#install(java.util.List<File>)
     */
    public void install(List<File> listFiles) {
        log.call();
        if (listFiles != null) {
            List<File> slToInstall = getSharedLibraryFileList(listFiles);
            for (File file : slToInstall) {
                try {
                    performInstallSL(file);
                } catch (Throwable e) {
                    log.error("Error during the installation of a shared library: ", e);
                } finally {
                    try {
                        listFiles.remove(file);
                        file.delete();
                    } catch (Throwable e) {
                        log.error("Error during the file destruction: ", e);
                    }
                }
            }
            List<File> compToInstall = getComponentFileList(listFiles);
            for (File file : compToInstall) {
                try {
                    performInstallComponent(file);
                } catch (Throwable e) {
                    log.error("Error during the installation of a component: ", e);
                } finally {
                    try {
                        listFiles.remove(file);
                        file.delete();
                    } catch (Throwable e) {
                        log.error("Error during the file destruction: ", e);
                    }
                }
            }
            List<File> saToInstall = getServiceAssemblyFileList(listFiles);
            for (File file : saToInstall) {
                try {
                    performDeploySA(file);
                } catch (Throwable e) {
                    log.error("Error during the deployment of a service assembly: ", e);
                } finally {
                    try {
                        listFiles.remove(file);
                        file.delete();
                    } catch (Throwable e) {
                        log.error("Error during the file destruction: ", e);
                    }
                }
            }
            for (File file : listFiles) {
                try {
                    log.info("A residual non jbi file was detected into the auto load directory: "
                            + file.getName() + ". Cleaner running...");
                    file.delete();
                } catch (Throwable e) {
                    log.error("Error during the file destruction: ", e);
                }
            }
        }
    }

    /**
     * Get the service assemblies file list into a global list<br />
     * (with shared libraries, components and service assemblies)
     * @param newInstallList The global list
     * @return The list of service assemblies
     */
    private List<File> getServiceAssemblyFileList(List<File> newInstallList) {
        File file = null;
        Jbi descriptor = null;
        List<File> list = new ArrayList<File>();
        Iterator<File> iterator = newInstallList.iterator();
        if (iterator != null) {
            while (iterator.hasNext()) {
                file = iterator.next();
                try {
                    descriptor = packageHelper.loadDescriptor(file.toURI());
                    if (descriptor.getServiceAssembly() != null) {
                        list.add(file);
                    }
                } catch (PetalsException e) {
                    log.warning("Unable to read a jbi descriptor: ", e);
                }
            }
        }
        return list;
    }

    /**
     * Get the components file list into a global list<br />
     * (with shared libraries, components and service assemblies)
     * @param newInstallList The global list
     * @return The list of components
     */
    private List<File> getComponentFileList(List<File> newInstallList) {
        File file = null;
        Jbi descriptor = null;
        List<File> list = new ArrayList<File>();
        Iterator<File> iterator = newInstallList.iterator();
        if (iterator != null) {
            while (iterator.hasNext()) {
                file = iterator.next();
                try {
                    descriptor = packageHelper.loadDescriptor(file.toURI());
                    if (descriptor.getComponent() != null) {
                        list.add(file);
                    }
                } catch (PetalsException e) {
                    log.warning("Unable to read a jbi descriptor: ", e);
                }
            }
        }
        return list;
    }

    /**
     * Get the shared libraries file list into a global list<br />
     * (with shared libraries, components and service assemblies)
     * @param newInstallList The global list
     * @return The list ofshared libraries
     */
    private List<File> getSharedLibraryFileList(List<File> newInstallList) {
        File file = null;
        Jbi descriptor = null;
        List<File> list = new ArrayList<File>();
        Iterator<File> iterator = newInstallList.iterator();
        if (iterator != null) {
            while (iterator.hasNext()) {
                file = iterator.next();
                try {
                    descriptor = packageHelper.loadDescriptor(file.toURI());
                    if (descriptor.getSharedLibrary() != null) {
                        list.add(file);
                    }
                } catch (PetalsException e) {
                    log.warning("Unable to read a jbi descriptor: ", e);
                }
            }
        }
        return list;
    }

    /**
     * @see org.ow2.petals.jbi.management.autoload.AutoLoaderService#uninstall(java
     *      .util.Collection)
     */
    public void uninstall(Collection<File> filesToUninstall) {
        log.call();
        try {
            /*
             * Check if one of the files to uninstall match one of the
             * archiveUrl of the installed entities. If yes, this entity must be
             * uninstalled.
             */
            List<String> compToUninstall = getComponentStateNameList(filesToUninstall);
            List<String> slToUninstall = getSharedLibraryStateNameList(filesToUninstall);
            List<String> saToUninstall = getServiceAssemblyStateNameList(filesToUninstall);
            // Perform entity uninstallation
            for (String name : saToUninstall) {
                performUndeploySA(name);
            }
            for (String name : compToUninstall) {
                performUninstallComponent(name);
            }
            for (String name : slToUninstall) {
                performUninstallSL(name);
            }
        } catch (Throwable e) {
            log.error("Error during uninstallation of JBI artifact(s)", e);
        }

    }

   
    /**
     * Get the service assemblies name list into a global list<br />
     * (with shared libraries, components and service assemblies)
     * @param fileCollection The global list
     * @return The list of service assemblies
     * @throws PetalsException Unable to retrieve the state of a service assembly
     */
    private List<String> getServiceAssemblyStateNameList(Collection<File> fileCollection)
            throws PetalsException {
        List<String> serviceAssemblyStateNameList = new ArrayList<String>();
        if (systemStateService != null) {
            List<ServiceAssemblyState> serviceAssemblyStateList = systemStateService
                    .retrieveAllServiceAssemblyStates();
            for (File file : fileCollection) {
                String serviceAssemblyStateName = getServiceAssemblyName(serviceAssemblyStateList,
                        file);
                if (serviceAssemblyStateName != null) {
                    serviceAssemblyStateNameList.add(serviceAssemblyStateName);
                }
            }
        }
        return serviceAssemblyStateNameList;
    }

    /**
     * Get the shared libraries name list into a global list<br />
     * (with shared libraries, components and service assemblies)
     * @param fileCollection The global list
     * @return The list of shared libraries
     * @throws PetalsException Unable to retrieve the state of a shared library
     */
    private List<String> getSharedLibraryStateNameList(Collection<File> fileCollection)
            throws PetalsException {
        List<String> sharedLibraryStateNameList = new ArrayList<String>();
        if (systemStateService != null) {
            List<SharedLibraryState> sharedLibraryStateList = systemStateService
                    .retrieveAllSharedLibraryStates();
            for (File file : fileCollection) {
                String sharedLibraryStateName = getSharedLibraryName(sharedLibraryStateList, file);
                if (sharedLibraryStateName != null) {
                    sharedLibraryStateNameList.add(sharedLibraryStateName);
                }
            }
        }
        return sharedLibraryStateNameList;
    }

    /**
     * Get the components name list into a global list<br />
     * (with shared libraries, components and service assemblies)
     * @param fileCollection The global list
     * @return The list of components
     * @throws PetalsException Unable to retrieve the state of a component
     */
    private List<String> getComponentStateNameList(Collection<File> fileCollection)
            throws PetalsException {
        List<String> componentStateNameList = new ArrayList<String>();
        if (systemStateService != null) {
            List<ComponentState> componentStateList = systemStateService
                    .retrieveAllComponentStates();
            for (File file : fileCollection) {
                String componentStateName = getComponentStateName(componentStateList, file);
                if (componentStateName != null) {
                    componentStateNameList.add(componentStateName);
                }
            }
        }
        return componentStateNameList;
    }

    /**
     * Get the service assembly name of an installed archive file into the service assemblies repository<br />
     * @param serviceAssemblyStateList The list of service assemblies into the repository
     * @param serviceAssemblyFile The installed archive file
     * @return The name of the service assembly or null if not found
     * @throws PetalsException Unable to retrieve the state name of a service assembly
     */
    private String getServiceAssemblyName(List<ServiceAssemblyState> serviceAssemblyStateList,
            File serviceAssemblyFile) throws PetalsException {
        for (ServiceAssemblyState state : serviceAssemblyStateList) {
            URI archiveURI = packageHelper.getPackageURI(state.getArchiveURL(), false);
            File archiveFile = new File(archiveURI.normalize());
            if (serviceAssemblyFile.equals(archiveFile)) {
                return state.getName();
            }
        }
        return null;
    }

    /**
     * Get the shared library name of an installed archive file into the shared libraries repository<br />
     * @param sharedLibraryStateList The list of shared libraries into the repository
     * @param sharedLibraryFile The installed archive file
     * @return The name of the shared library or null if not found
     * @throws PetalsException Unable to retrieve the state name of a shared library
     */
    private String getSharedLibraryName(List<SharedLibraryState> sharedLibraryStateList,
            File sharedLibraryFile) throws PetalsException {
        for (SharedLibraryState state : sharedLibraryStateList) {
            URI archiveURI = packageHelper.getPackageURI(state.getArchiveURL(), false);
            File archiveFile = new File(archiveURI.normalize());
            if (sharedLibraryFile.equals(archiveFile)) {
                return state.getName();
            }
        }
        return null;
    }

    /**
     * Get the component name of an installed archive file into the components repository<br />
     * @param componentStateList The list of components into the repository
     * @param componentFile The installed archive file
     * @return The name of the component or null if not found
     * @throws PetalsException Unable to retrieve the state name of a component
     */
    private String getComponentStateName(List<ComponentState> componentStateList, File componentFile)
            throws PetalsException {
        for (ComponentState state : componentStateList) {
            URI archiveURI = packageHelper.getPackageURI(state.getArchiveURL(), false);
            File archiveFile = new File(archiveURI.normalize());
            if (componentFile.equals(archiveFile)) {
                return state.getName();
            }
        }
        return null;
    }

    /**
     * Install the specified Archive in the JBI container and start it . Any
     * exception will cause the abortion of the process. A file
     * "fileName.success" will be created if success A file "fileName.failed"
     * will be created otherwise, containing the related exception
     * 
     * @param componentFile The component
     */
    protected void performInstallComponent(File componentFile) {
        log.call();
        if (componentFile != null && componentFile.exists()) {
            try {
                MBeanServer jmxServer = jmxService.getLocalJMXServer();

                ObjectName newInstallerName = installationService.loadNewInstaller(componentFile
                        .toURI().toString());
                if (!newInstallerName.getDomain().equals("error")) {
                    Object[] nullObjects = new Object[0];
                    String[] nullStrings = new String[0];
                    Object result = jmxServer.invoke(newInstallerName, "install", nullObjects,
                            nullStrings);
                    if (result instanceof ObjectName) {
                        ObjectName componentName = (ObjectName) result;
                        jmxServer.invoke(componentName, "start", nullObjects, nullStrings);
                    } else {
                        throw new Exception(
                                "The install() invokation on the component installer has returned an unexpected object: "
                                        + result);
                    }
                } else {
                    throw new Exception("Component can't be auto installed");
                }
            } catch (Throwable e) {
                log.error("Error occured during auto installation: ", e);
                createFailureFile(componentFile, e);
            }
        }
    }

    /**
     * Uninstall the specified component.
     * 
     * @param name The component
     */
    protected void performUninstallComponent(String name) {
        installationService.forceUnloadComponentInstaller(name);
    }

    /**
     * Deploy the specified Archive in the JBI container and start it . Any
     * exception will cause the abortion of the process. A file
     * "fileName.success" will be created if success A file "fileName.failed"
     * will be created otherwise, containing the related exception
     * 
     * @param saFile the archive file
     */
    protected void performDeploySA(File saFile) {
        log.call();

        if (saFile != null && saFile.exists()) {

            try {
                String xml = deploymentService.deploy(saFile.toURI().toString());

                if (xml.indexOf(XMLResult.TaskResult.FAILED.toString()) > -1) {
                    log.warning("Problem while deploying a service assembly\n" + xml);
                } else {
                    String saName = "";
                    saName = xml.substring(xml.indexOf("<loc-param>") + 11, xml
                            .indexOf("</loc-param>"));
                    deploymentService.start(saName);

                }
            } catch (Throwable e) {
                log.error("Error occured during auto deployment", e);
                createFailureFile(saFile, e);
            }
        }
    }

    /**
     * Undeploy the specified service assembly. isToBeDeleted is setted to true
     * !
     * 
     * @param name The service assembly
     */
    protected void performUndeploySA(String name) {
        deploymentService.forceUndeploy(name);
    }

    /**
     * Install the specified shared library in the JBI container Any exception
     * will cause the abortion of the process. A file "fileName.success" will be
     * created if success A file "fileName.failed" will be created otherwise,
     * containing the related exception
     * 
     * @param slFile The shared library
     */
    protected void performInstallSL(File slFile) {
        log.call();

        if (slFile != null && slFile.exists()) {

            try {
                installationService.installSharedLibrary(slFile.toURI().toString());

            } catch (Throwable e) {
                createFailureFile(slFile, e);
                log.error("Error occured during auto Shared library installation.", e);
            }
        }
    }

    /**
     * Uninstall a shared library.
     * 
     * @param name The shared library
     */
    protected void performUninstallSL(String name) {
        installationService.uninstallSharedLibrary(name);
    }

    /**
     * Create an error report in the work directory.
     * 
     * @param file
     * @param exception
     * @throws IOException
     * @throws FileNotFoundException
     */
    private void createFailureFile(File file, Throwable exception) {
        File failed = new File(workDirectory, file.getName() + ".failed");
        try {
            failed.createNewFile();
            PrintWriter w = new PrintWriter(failed);
            exception.printStackTrace(w);
            w.flush();
            w.close();
        } catch (FileNotFoundException e) {
            log.error("Error occured during failure report creation.", e);
        } catch (IOException e) {
            log.error("Error occured during failure report creation.", e);
        }
    }

    /**
     * For each entity to uninstall, search the linked entities<br />
     * and add them to the collection of entity. A shared library can have components linked to,<br />
     * and a component can have service assemblies linked to.<br />
     * The method has a robust approach : if we are not able to create the linked collection of files<br />
     * (bacause of an exception or a null value), we considere that it doesn't exist linked files for this file. 
     * 
     * @param filesToUninstall The initial collection of files to uninstall 
     * @return A Collection of file with the intials and the linked files to uninstall
     */
    public Collection<File> getFileCollectionWithLinkedFileCollectionToUninstall(
            Collection<File> filesToUninstall) {
        Set<File> linkedFileSet = new HashSet<File>();
        List<ComponentState> ComponentStatelist = null;
        List<ServiceAssemblyState> serviceAssemblyStateList = null;
        try {
            /*
             * Check if one of the files to uninstall match one of the
             * archiveUrl of the installed entities. If yes, this entity must be
             * checked to found linked entities to uninstall too.
             */
            // Perform components checking
            List<String> compToUninstall = getComponentStateNameList(filesToUninstall);
            for (String name : compToUninstall) {
                // check the linked service assemblies
                if (serviceAssemblyStateList == null) {
                    serviceAssemblyStateList = systemStateService
                            .retrieveAllServiceAssemblyStates();
                }
                for (ServiceAssemblyState state : serviceAssemblyStateList) {
                    if (state != null) {
                        // Load component JBI
                        // descriptor
                        Jbi serviceAssemblyDescriptor = packageHelper.loadDescriptor(state
                                .getName(), EntityType.SA_TYPE);
                        if (serviceAssemblyDescriptor != null
                                && serviceAssemblyDescriptor.getServiceAssembly() != null) {
                            List<ServiceUnit> serviceUnitList = serviceAssemblyDescriptor
                                    .getServiceAssembly().getServiceUnit();
                            if (serviceUnitList != null) {
                                Iterator<ServiceUnit> serviceUnitIterator = serviceUnitList
                                        .iterator();
                                if (serviceUnitIterator != null) {
                                    boolean found = false;
                                    while (!found && serviceUnitIterator.hasNext()) {
                                        ServiceUnit serviceUnit = serviceUnitIterator.next();
                                        if (serviceUnit != null && serviceUnit.getTarget() != null) {
                                            String componentName = serviceUnit.getTarget()
                                                    .getComponentName();
                                            if (componentName != null && componentName.equals(name)) {
                                                URI archiveURI = packageHelper.getPackageURI(state
                                                        .getArchiveURL(), false);
                                                File file = new File(archiveURI.normalize());
                                                linkedFileSet.add(file);
                                                found = true;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
            // Perform shared libraries checking
            List<String> slToUninstall = getSharedLibraryStateNameList(filesToUninstall);
            for (String name : slToUninstall) {
                // check the linked components
                if (ComponentStatelist == null) {
                    ComponentStatelist = systemStateService.retrieveAllComponentStates();
                }
                for (ComponentState state : ComponentStatelist) {
                    if (state != null) {

                        Jbi componentDescriptor = packageHelper.loadDescriptor(state.getName(),
                                EntityType.COMPONENT_TYPE);
                        if (componentDescriptor != null
                                && componentDescriptor.getComponent() != null) {
                            List<SharedLibrary> sls = componentDescriptor.getComponent()
                                    .getSharedLibraryList();
                            if (sls != null) {
                                Iterator<SharedLibrary> sharedLibraryIterator = sls.iterator();
                                if (sharedLibraryIterator != null) {
                                    boolean found = false;
                                    while (!found && sharedLibraryIterator.hasNext()) {
                                        SharedLibrary sharedLibrary = sharedLibraryIterator.next();
                                        if (sharedLibrary != null) {
                                            String sharedLibraryName = sharedLibrary.getContent();
                                            if (sharedLibraryName.equals(name)) {
                                                URI archiveURI = packageHelper.getPackageURI(state
                                                        .getArchiveURL(), false);
                                                File file = new File(archiveURI.normalize());
                                                linkedFileSet.add(file);
                                            }
                                        }
                                    }

                                }
                            }
                        }

                    }
                }
            }
        } catch (Throwable e) {
            log.error("Error during checking linked JBI artifact(s)", e);
        }
        if (!linkedFileSet.isEmpty()) {
            Collection<File> fileSet = getFileCollectionWithLinkedFileCollectionToUninstall(linkedFileSet);
            linkedFileSet.addAll(fileSet);
        }
        filesToUninstall.addAll(linkedFileSet);
        return filesToUninstall;
    }

}
