/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.net.URI;
import java.util.HashMap;

import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task creates state holder entry for component, service assembly or
 * shared lib <br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * Inputs : {@link Jbi}, archive {@link URI}
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public abstract class AbstractCreateStateHolderTask extends AbstractFileManipulationTask {

    /**
     * JBI Container :: Recovery Service
     */
    protected SystemStateService systemStateSrv = null;

    /**
     * Platform Component :: Repository Service
     */
    protected RepositoryService repositorySrv;

    public AbstractCreateStateHolderTask(LoggingUtil log, SystemStateService systemStateSrv,
            RepositoryService repositorySrv, String petalsInstallDirectory) {
        super(log, petalsInstallDirectory);
        this.systemStateSrv = systemStateSrv;
        this.repositorySrv = repositorySrv;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);
        if (descriptor == null) {
            throw new PetalsException("Error creating the descriptor for the stateholder.");
        }
        URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);
        if (archiveURI == null) {
            throw new PetalsException("Error creating the archive URI for the stateholder.");
        }
        File file = getArchiveFile(archiveURI);
        if (file == null) {
            throw new PetalsException("Error creating the archive file for the stateholder.");
        }
        File installedArchive = getInstalledArchive(file);
        if (installedArchive == null) {
            throw new PetalsException(
                    "Error creating the installed archive file for the stateholder.");
        }
        createEntityStateHolder(descriptor, installedArchive);
    }

    protected abstract void createEntityStateHolder(Jbi descriptor, File installedArchive)
            throws Exception;

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        if (context.containsKey(TaskContextConstants.DESCRIPTOR)) {
            Jbi descriptor = (Jbi) context.get(TaskContextConstants.DESCRIPTOR);
            if (descriptor != null) {
                try {
                    deleteEntityStateHolder(descriptor);
                } catch (Exception e) {
                    log.error("Failed to revert a CreateStateHolderTask", e);
                }
            }
        }
    }

    protected abstract void deleteEntityStateHolder(Jbi descriptor) throws Exception;

}
