/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.util.HashMap;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task download entity (component, shared lib, service assembly) archive
 * into the work directory, only if the URL protocol is http: or ftp: <br/>
 * Inputs : archive {@link URI} <br/>
 * Outputs : downloaded archive {@link URI}<br/>
 * The task has got a non-robust approach : it stops at the first subtask failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class DownloadTask extends AbstractFileManipulationTask {

    public DownloadTask(LoggingUtil log, String petalsInstallDirectory) {
        super(log, petalsInstallDirectory);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);
        // Explode Comp
        URI workArchiveURI = this.downloadEntityPackage(archiveURI);
        // Fill context
        context.put(TaskContextConstants.ARCHIVE_URI, workArchiveURI);
    }

    private URI downloadEntityPackage(URI archiveURI) throws PetalsException {
        InputStream inputStream = null;
        FileOutputStream fileOutputStream = null;
        File outputFile = null;
        URI uri = null;
        try {
            String fileName = archiveURI.toURL().getFile().substring(
                    archiveURI.toURL().getFile().lastIndexOf('/') + 1);
            inputStream = archiveURI.toURL().openStream();
            outputFile = new File(this.getWorkDirectory(), fileName);
            fileOutputStream = new FileOutputStream(outputFile);
            IOUtils.copy(inputStream, fileOutputStream);
            uri = outputFile.toURI();
        } catch (MalformedURLException e) {
            throw new PetalsException("Can't read distant file: " + archiveURI, e);
        } catch (IOException e) {
            throw new PetalsException("Can't read distant file: " + archiveURI, e);
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
                if (fileOutputStream != null) {
                    fileOutputStream.close();
                }
            } catch (IOException e) {
                log.error("Unable to close the I/O operators to download", e);
            }
        }
        return uri;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        if (context.containsKey(TaskContextConstants.ARCHIVE_URI)) {
            URI downloadURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);
            if (downloadURI != null) {
                try {
                    FileUtils.forceDelete(new File(downloadURI));
                } catch (IOException e) {
                    log.error("Failed to clean downloaded artefact " + downloadURI);
                }
            }
        }
    }

}
