/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.undeploy;

import java.io.File;
import java.net.URI;
import java.util.HashMap;

import org.ow2.petals.jbi.management.task.AbstractFileManipulationTask;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task deletes the service assembly archive from the installed dir of the
 * Petals repository <br/>
 * Inputs : archive {@link URI}<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * 
 * @author ofabre
 * @author Frederic Gardes - eBM Websourcing
 */
public class RemoveSAArchiveTask extends AbstractFileManipulationTask {

    public RemoveSAArchiveTask(LoggingUtil log, String petalsInstallDirectory) {
        super(log, petalsInstallDirectory);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        if (context.containsKey(TaskContextConstants.ARCHIVE_URI)) {
            URI archiveURI = (URI) context.get(TaskContextConstants.ARCHIVE_URI);
            if (archiveURI != null) {
                File file = getArchiveFile(archiveURI);
                if (file != null) {
                    File installedArchive = getInstalledArchive(file);
                    if (installedArchive != null) {
                        installedArchive.delete();
                    }
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        // TODO : recreate the sa archive
    }

}
