/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: RemoveSAStateHolderTask.java 154 10 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.deployment.undeploy;

import java.util.HashMap;

import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task removes service assembly from state holder file<br/>
 * Inputs : entity name {@link String}<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class RemoveSAStateHolderTask extends AbstractLoggableTask {

    /**
     * JBI Container :: Recovery Service
     */
    private SystemStateService recoverySrv;

    public RemoveSAStateHolderTask(LoggingUtil log, SystemStateService recoverySrv) {
        super(log);
        this.recoverySrv = recoverySrv;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        if (context.containsKey(TaskContextConstants.ENTITY_NAME)) {
            String name = (String) context.get(TaskContextConstants.ENTITY_NAME);
            if (name != null) {
                try {
                    recoverySrv.deleteServiceAssemblyStateHolder(name);
                } catch (Exception e) {
                    log.error("The service assembly " + name + " state holder can't be removed: ",
                            e);
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        // TODO : recreate the service assembly state holder
    }

}
