/**
 * PETALS - PETALS Services Platform. Copyright (c) 2006 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: InstallationTaskFactory.java 154 6 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation;

import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.container.ContainerService;
import org.ow2.petals.jbi.management.admin.AdminServiceMBean;
import org.ow2.petals.jbi.management.task.CopyArchiveToInstalledDirTask;
import org.ow2.petals.jbi.management.task.CopyArchiveToWorkDirTask;
import org.ow2.petals.jbi.management.task.DownloadTask;
import org.ow2.petals.jbi.management.task.ExtractTask;
import org.ow2.petals.jbi.management.task.SuccessTask;
import org.ow2.petals.jbi.management.task.installation.install.component.CheckInstalledSLForCompTask;
import org.ow2.petals.jbi.management.task.installation.install.component.ComponentCheckPackageTask;
import org.ow2.petals.jbi.management.task.installation.install.component.CreateAndRegisterInstallerMBeanTask;
import org.ow2.petals.jbi.management.task.installation.install.component.CreateComponentFolderTask;
import org.ow2.petals.jbi.management.task.installation.install.component.CreateComponentStateHolderTask;
import org.ow2.petals.jbi.management.task.installation.install.sl.CopySLPackageToRepositoryTask;
import org.ow2.petals.jbi.management.task.installation.install.sl.CreateSLClassloaderTask;
import org.ow2.petals.jbi.management.task.installation.install.sl.CreateSLStateHolderTask;
import org.ow2.petals.jbi.management.task.installation.install.sl.SLCheckPackageTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.RemoveComponentArchiveTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.RemoveComponentFolderTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.RemoveComponentStateHolderTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.component.UnregisterInstallerTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.sl.RemoveSLArchiveTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.sl.RemoveSLClassloaderTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.sl.RemoveSLFolderTask;
import org.ow2.petals.jbi.management.task.installation.uninstall.sl.RemoveSLStateHolderTask;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.jbi.messaging.routing.Router;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.platform.systemstate.SystemStateService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This Factory creates Installation Tasks.
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class InstallationTaskFactory {
    private CopyArchiveToWorkDirTask copyArchiveToWorkDirTask;

    private CheckInstalledSLForCompTask checkInstalledSLForCompTask;

    private CreateComponentFolderTask createComponentFolderTask;

    private ComponentCheckPackageTask componentCheckPackageTask;

    private CreateComponentStateHolderTask createComponentStateHolderTask;

    private CreateAndRegisterInstallerMBeanTask createAndRegisterInstallerMBeanTask;

    private CreateSLClassloaderTask createSLClassloaderTask;

    private CopySLPackageToRepositoryTask copySLPackageToRepositoryTask;

    private SLCheckPackageTask slCheckPackageTask;

    private CreateSLStateHolderTask createSLStateHolderTask;

    private CopyArchiveToInstalledDirTask copyArchiveToInstalledDirTask;

    private RemoveComponentFolderTask removeComponentFolderTask;

    private RemoveComponentStateHolderTask removeComponentStateHolderTask;

    private RemoveComponentArchiveTask removeComponentArchiveTask;

    private UnregisterInstallerTask unregisterInstallerTask;

    private RemoveSLArchiveTask removeSLArchiveTask;

    private RemoveSLClassloaderTask removeSLClassloaderTask;

    private RemoveSLFolderTask removeSLFolderTask;

    private RemoveSLStateHolderTask removeSLStateHolderTask;

    private SuccessTask successTask;

    private ExtractTask extractTask;

    private DownloadTask downloadTask;

    /**
     * Creates a new instance of {@link InstallationTaskFactory}
     * 
     * @param packageHandler
     * @param systemStateService
     * @param repositoryService
     * @param log
     * @param logger
     * @param adminService
     * @param router
     * @param loaderService
     */
    public InstallationTaskFactory(PackageHelper packageHandler,
            SystemStateService systemStateService, RepositoryService repositoryService,
            LoggingUtil log, Logger logger, AdminServiceMBean adminService, Router router,
            ContainerService container, String petalsInstallDirectory) {
        super();

        /*
         * Create Tasks
         */
        // Component creation tasks
        checkInstalledSLForCompTask = new CheckInstalledSLForCompTask(systemStateService);

        createComponentFolderTask = new CreateComponentFolderTask(log, repositoryService);

        componentCheckPackageTask = new ComponentCheckPackageTask(log, packageHandler, adminService);

        createComponentStateHolderTask = new CreateComponentStateHolderTask(log,
                systemStateService, repositoryService, petalsInstallDirectory);

        createAndRegisterInstallerMBeanTask = new CreateAndRegisterInstallerMBeanTask(log,
                adminService, container);

        successTask = new SuccessTask(log, petalsInstallDirectory);

        // SL creation tasks
        createSLClassloaderTask = new CreateSLClassloaderTask(log, adminService, container);

        copySLPackageToRepositoryTask = new CopySLPackageToRepositoryTask(log, repositoryService);

        slCheckPackageTask = new SLCheckPackageTask(log, packageHandler, adminService);

        createSLStateHolderTask = new CreateSLStateHolderTask(log, systemStateService,
                repositoryService, petalsInstallDirectory);

        copyArchiveToInstalledDirTask = new CopyArchiveToInstalledDirTask(log,
                petalsInstallDirectory);

        // Component removal tasks
        removeComponentFolderTask = new RemoveComponentFolderTask(log, repositoryService);

        removeComponentStateHolderTask = new RemoveComponentStateHolderTask(log, systemStateService);

        removeComponentArchiveTask = new RemoveComponentArchiveTask(log, petalsInstallDirectory,
                packageHandler);

        unregisterInstallerTask = new UnregisterInstallerTask(log, adminService, container);

        // SL removal tasks
        removeSLArchiveTask = new RemoveSLArchiveTask(log, petalsInstallDirectory, packageHandler);

        removeSLClassloaderTask = new RemoveSLClassloaderTask(log, adminService, container);

        removeSLFolderTask = new RemoveSLFolderTask(log, repositoryService);

        removeSLStateHolderTask = new RemoveSLStateHolderTask(log, systemStateService);

        // shared tasks
        extractTask = new ExtractTask(log, petalsInstallDirectory);

        downloadTask = new DownloadTask(log, petalsInstallDirectory);

        copyArchiveToWorkDirTask = new CopyArchiveToWorkDirTask(log, petalsInstallDirectory);
    }

    public DownloadTask getDownloadTask() {
        return downloadTask;
    }

    public CopyArchiveToWorkDirTask getCopyArchiveToWorkDirTask() {
        return copyArchiveToWorkDirTask;
    }

    public CheckInstalledSLForCompTask getCheckInstalledSLForCompTask() {
        return checkInstalledSLForCompTask;
    }

    public CreateComponentFolderTask getCreateComponentFolderTask() {
        return createComponentFolderTask;
    }

    public ComponentCheckPackageTask getComponentCheckPackageTask() {
        return componentCheckPackageTask;
    }

    public CreateComponentStateHolderTask getCreateComponentStateHolderTask() {
        return createComponentStateHolderTask;
    }

    public CopyArchiveToInstalledDirTask getCopyArchiveToInstalledDirTask() {
        return copyArchiveToInstalledDirTask;
    }

    public CreateAndRegisterInstallerMBeanTask getCreateAndRegisterInstallerMBeanTask() {
        return createAndRegisterInstallerMBeanTask;
    }

    public CreateSLClassloaderTask getCreateSLClassloaderTask() {
        return createSLClassloaderTask;
    }

    public CopySLPackageToRepositoryTask getCopySLPackageToRepositoryTask() {
        return copySLPackageToRepositoryTask;
    }

    public SLCheckPackageTask getSLCheckPackageTask() {
        return slCheckPackageTask;
    }

    public CreateSLStateHolderTask getCreateSLStateHolderTask() {
        return createSLStateHolderTask;
    }

    public RemoveComponentArchiveTask getRemoveComponentArchiveTask() {
        return removeComponentArchiveTask;
    }

    public RemoveSLArchiveTask getRemoveSLArchiveTask() {
        return removeSLArchiveTask;
    }

    public UnregisterInstallerTask getUnregisterInstallerTask() {
        return unregisterInstallerTask;
    }

    public RemoveSLClassloaderTask getRemoveSLClassloaderTask() {
        return removeSLClassloaderTask;
    }

    public RemoveSLFolderTask getRemoveSLFolderTask() {
        return removeSLFolderTask;
    }

    public RemoveSLStateHolderTask getRemoveSLStateHolderTask() {
        return removeSLStateHolderTask;
    }

    public RemoveComponentFolderTask getRemoveComponentFolderTask() {
        return removeComponentFolderTask;
    }

    public RemoveComponentStateHolderTask getRemoveComponentStateHolderTask() {
        return removeComponentStateHolderTask;
    }

    public SuccessTask getSuccessTask() {
        return successTask;
    }

    public ExtractTask getExtractTask() {
        return extractTask;
    }

}
