/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.uninstall.component;

import java.io.File;
import java.net.URI;
import java.util.HashMap;

import org.ow2.petals.jbi.management.task.AbstractFileManipulationTask;
import org.ow2.petals.jbi.management.task.installation.InstallationContextConstants;
import org.ow2.petals.jbi.management.util.PackageHelper;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.platform.systemstate.ComponentState;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task moves a component archive from the installed Petals directory to
 * the uninstalled directory<br/>
 * Inputs : {@link ComponentState}<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * 
 * @author ofabre
 * @author Frederic Gardes - eBM Websourcing
 */
public class RemoveComponentArchiveTask extends AbstractFileManipulationTask {

    private PackageHelper packageHelper = null;

    public RemoveComponentArchiveTask(LoggingUtil log, String petalsInstallDirectory,
            PackageHelper packageHelper) {
        super(log, petalsInstallDirectory);
        this.packageHelper = packageHelper;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        File installedArchive = null;
        if (context.containsKey(InstallationContextConstants.COMPONENT_STATE)) {
            ComponentState componentState = (ComponentState) context
                    .get(InstallationContextConstants.COMPONENT_STATE);
            if (componentState != null) {
                try {
                    URI archiveURI = packageHelper.getPackageURI(componentState.getArchiveURL(),
                            false);
                    installedArchive = new File(archiveURI);
                    if (installedArchive != null && installedArchive.exists()) {
                        installedArchive.delete();
                    }
                } catch (PetalsException e) {
                    log.error("The archive can't be removed: ", e);
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        // TODO : recreate the component archive
    }
}
