/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.uninstall.component;

import java.util.HashMap;

import org.ow2.petals.jbi.management.task.AbstractLoggableTask;
import org.ow2.petals.jbi.management.task.TaskContextConstants;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.platform.repository.RepositoryService;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task remove the component folder from the Petals repository<br/>
 * Inputs : component name<br />
 * The task has got a robust approach : it tries to do all the subtasks<br />
 * and doesn't have the undo capability.<br />
 * 
 * @author ofabre
 * @author Frederic Gardes - eBM Websourcing
 */
public class RemoveComponentFolderTask extends AbstractLoggableTask {

    private RepositoryService repositoryService;

    public RemoveComponentFolderTask(LoggingUtil log, RepositoryService repositoryService) {
        super(log);
        this.repositoryService = repositoryService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#execute(java.util.HashMap)
     */
    public void execute(HashMap<String, Object> context) throws Exception {
        if (context.containsKey(TaskContextConstants.ENTITY_NAME)) {
            String componentName = (String) context.get(TaskContextConstants.ENTITY_NAME);
            if (componentName != null) {
                try {
                    repositoryService.removeComponentPackage(componentName);
                } catch (PetalsException e) {
                    log.error("The component " + componentName + " folder can't be removed: ", e);
                }
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ow2.petals.processor.Task#undo(java.util.HashMap)
     */
    public void undo(HashMap<String, Object> context) throws Exception {
        // TODO : recreate the component folder
    }

}
