/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.platform.persistence;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.Serializable;
import java.util.Map;
import java.util.Properties;

import net.sf.joafip.java.util.PHashMap;
import net.sf.joafip.service.FilePersistenceException;

import org.jgroups.persistence.CannotPersistException;
import org.jgroups.persistence.CannotRemoveException;
import org.jgroups.persistence.CannotRetrieveException;
import org.jgroups.persistence.PersistenceManager;

/**
 * Wrap the JOAFIP persistence manager to be usable in Jgroups persistence.
 * 
 * @author rnaudin
 * 
 */
public class FilePersistenceManager implements PersistenceManager {

    private static final String PERSIST_PROPERTY = "persist";

    private JNDIPersistentMap<Serializable, PHashMap<String, Object>> map;

    /**
     * Default constructor
     * 
     * @throws IOException
     * @throws FilePersistenceException
     * @throws FilePersistenceException
     * @throws FileNotFoundException
     */
    public FilePersistenceManager(String propertiesFilename) throws IOException,
            FilePersistenceException {
        Properties properties = new Properties();
        properties.load(new FileInputStream(propertiesFilename));
        String path = properties.getProperty(PERSIST_PROPERTY);
        this.map = new JNDIPersistentMap<Serializable, PHashMap<String, Object>>(new File(path));
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.jgroups.persistence.PersistenceManager#clear()
     */
    public void clear() throws CannotRemoveException {
        this.map.clear();
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.jgroups.persistence.PersistenceManager#remove(java.io.Serializable)
     */
    public Serializable remove(Serializable key) throws CannotRemoveException {
        return this.map.remove(key);

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.jgroups.persistence.PersistenceManager#retrieveAll()
     */
    @SuppressWarnings("unchecked")
    public Map retrieveAll() throws CannotRetrieveException {
        return this.map;
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.jgroups.persistence.PersistenceManager#save(java.io.Serializable,
     * java.io.Serializable)
     */
    @SuppressWarnings("unchecked")
    public void save(Serializable key, Serializable value) throws CannotPersistException {
        this.map.put(key, (PHashMap<String, Object>) value);

    }

    /*
     * (non-Javadoc)
     * 
     * @see org.jgroups.persistence.PersistenceManager#saveAll(java.util.Map)
     */
    @SuppressWarnings("unchecked")
    public void saveAll(Map arg0) throws CannotPersistException {
        this.map.putAll(arg0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.jgroups.persistence.PersistenceManager#shutDown()
     */
    public void shutDown() {
        this.map.shutDown();
    }

}
